/** 
 *******************************************************************************
 * @file    com_control.c
 * @brief   UART control.
 * @version V1.1.0.0
 * $Date:: 2018-11-29
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2018 All rights reserved
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif


/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "stdio.h"

#include "system_TMPM3Hy.h"
#include "TMPM3HQ.h"

#include "txz_uart.h"
#include "txz_t32a.h"

#include "main.h"
#include "com_control.h"
#include "Flash_Erase.h"
#include "Flash_Write.h"


/*------------------------------------------------------------------------------*/
/*  Function Definitions                                                        */
/*------------------------------------------------------------------------------*/
void StartCom(void);
void ChkComMode(void);
void AutoBaud(void);
void StartUart(void);
void SendData(uint8_t *pData, uint32_t nLen);
uint32_t RecvData(uint8_t *pData, uint32_t nLen);
void TimerStart(void);
void TimerStop(void);


/*------------------------------------------------------------------------------*/
/*  Variable Definitions                                                        */
/*------------------------------------------------------------------------------*/
uint32_t	gtAB;				/* Timer count value between A and B. */
uint32_t	gtAC;				/* Timer count value between A and C. */
uint32_t	gtAD;				/* Timer count value between A and D. */
uint32_t	gtBC;				/* Timer count value between B and C. */
uint32_t	gtCD;				/* Timer count value between C and D. */
uint32_t	gtAC_rev;			/* gtAC after delay error correction. */

uint32_t	glb_N_value;		/* N value divided by 64. */
uint32_t	glb_K_value;		/* K value divided by 64. */


/*
 *********************************************************************
 * NAME	: StartCom
 *--------------------------------------------------------------------
 * DESCRIPTION	: ʐMJn
 * PARAMETER	: None.
 *
 * RETURN VALUE	: None.
 *
 *********************************************************************
 */
void StartCom(void)
{

	uint16_t i;


	/* LED1_ON */
	TSB_PK->DATA = (0x00000020UL);

	/* T32A related register setting. */
	do {
		TSB_T32A0->MOD = T32A_MODE_32;				/* T32Ax->MOD = 0x0000_0001 */
	} while (TSB_T32A0->MOD != T32A_MODE_32);		/* Read confirmation after writing. */

	TSB_T32A0->CRC = (0x00000000UL);				/* T32Ax->CRC = 0x0000_0000 */


	/* PORT_A fixed in TMPM3HQ. */
	/* UART RXD port pull-up and input setting. */
	TSB_PA->PUP |= (0x00000004UL);
	TSB_PA->IE |= (0x00000004UL);

	for (i = 0; i < 1680; i++) {;}					/* Wait until the setting is reflected. */

	for (;;) {
		if ((TSB_PA->DATA & (0x00000004UL)) == Low) {	/* UART ch.0 RXD port Low signal detection. */
			TimerStart();								/* T32A Timer-C start. */	
			TSB_PA->PUP &= (0xFFFFFFFBUL);				/* Pull-up release. */
			break;
		} else {
			/* Do Nothing */
		}
	}

	ChkComMode();		/* Command 'x'(0x78) Judgment process. */
	AutoBaud();			/* Baud rate automatic calculation processing. */
	StartUart();		/* UART communication start processing. */

	/* LED1_OFF */
	TSB_PK->DATA = (0x00000000UL);

	return;
}


/*
 *********************************************************************
 * NAME	: ChkComMode
 *--------------------------------------------------------------------
 * DESCRIPTION	: Command 'x'(0x78) Judgment process.
 * PARAMETER	: None.
 *
 * RETURN VALUE	: None.
 *
 *********************************************************************
 */
void ChkComMode(void)
{

	/* ABn_ o	*/
	for(;;) {
		if((TSB_PA->DATA & (0x00000004UL)) == 0x00000004) {
			break;								/* High signal detection. */
		}
		else{
			/* Do Nothing */
		}
	}
	gtAB = TSB_T32A0->TMRC;						/* Acquire 32-bit timer capture value between A and B. */

	/* ACn_ o	*/
	for(;;) {
		if((TSB_PA->DATA & (0x00000004UL)) == Low) {
			break;								/* Low signal detection. */
		}
		else{
			/* Do Nothing */
		}
	}
	gtAC = TSB_T32A0->TMRC;						/* Acquire 32-bit timer capture value between A and C. */

	/* ADn_ o	*/
	for(;;) {
		if((TSB_PA->DATA & (0x00000004UL)) == 0x00000004) {
			break;								/* High signal detection. */
		}
		else{
			/* Do Nothing */
		}
	}
	gtAD = TSB_T32A0->TMRC;						/* Acquire 32-bit timer capture value between A and D. */

	TimerStop();								/* T32A Timer-C stop. */


	gtBC = gtAC - gtAB;							/* Count value calculation between B and C. */
	gtCD = gtAD - gtAC;							/* Count value calculation between C and D. */


	/* Error detection. */
	if ((gtAB > (gtCD * 3)) && (gtAB < (gtCD * 5))) {
		/* Do Nothing */
	} else {
		BootStop();
	}
	

	if ((gtBC > (gtCD * 3)) && (gtBC < (gtCD * 5))) {
		/* Do Nothing */
	} else {
		BootStop();
	}

	return;

}


/*
 *********************************************************************
 * NAME	: AutoBaud
 *--------------------------------------------------------------------
 * DESCRIPTION	: Baud rate automatic calculation processing.
 * PARAMETER	: None.
 *
 * RETURN VALUE	: None.
 *
 *********************************************************************
 */
void AutoBaud(void)
{

	uint32_t	ResolIdx;				/* Decomposition index. */
	uint32_t	DivNum;					/* Number of minutes released .*/
	uint8_t		LoopCnt;				/* Decomposition index counter. */
	uint32_t	T0_Ct = 0;				/* Phi-T0 counter value. */
	uint32_t	Tn_Ct = 0;				/* Phi-Tn counter value. */

	glb_N_value = 0;
	glb_K_value = 0;

	/* Capture value correction. */
	gtAC_rev = gtAC + 12;				/* Worst half of timer start delay and timer stop delay. */

	/* Details of the calculation are confirmed in the design document. */
	ResolIdx = 0;						/* Decomposition index initialization. */
	DivNum = 1;							/* Initialization minutes released. */
	T0_Ct = gtAC_rev / 8;				/* Phi-T0 count value calculation. */

	for(LoopCnt = 0; LoopCnt < RESOL_MAX; LoopCnt++) {
		DivNum = 1 << ResolIdx;			/* Calculate minute release number. */
		Tn_Ct = T0_Ct / DivNum;			/* Phi-Tn count value calculation. */
		if(Tn_Ct < 16) {
			BootStop();
		} else {
			/* Do Nothing */
		}
		if(Tn_Ct <= (uint32_t)(65535 * 16)) {
			break;
		} else {
			/* Do Nothing */
		}
		ResolIdx++;
	}

	if(LoopCnt >= RESOL_MAX) {
		BootStop();
	} else {
		/* Do Nothing */
	}

	glb_N_value = Tn_Ct / 16;
	glb_K_value = (64 * glb_N_value) + 64 - (4 * Tn_Ct);
	if (glb_K_value == 64) {
		glb_N_value = glb_N_value - 1;
		glb_K_value = 0;
	} else {
		/* Do Nothing */
	}

	if (glb_N_value == 0) {
		BootStop();
	} else {
		/* Do Nothing */
	}

	return;

}


/*
 *********************************************************************
 * NAME	: StartUart
 *--------------------------------------------------------------------
 * DESCRIPTION	: UART communication start processing.
 * PARAMETER	: None.
 *
 * RETURN VALUE	: None.
 *
 *********************************************************************
 */
void StartUart(void)
{


	/* UART ch.0 port function setting and output setting. */
	TSB_PA->FR1 = (0x00000006UL);
	TSB_PA->CR  = (0x00000002UL);

	/* UART ch.0 related register setting. */
	TSB_UART0->CR0 = (0x00000001UL);
	TSB_UART0->CR1 = (0x00000150UL);
	TSB_UART0->CLK = (0x00000000UL);
	TSB_UART0->BRD = (0x08000000UL) | (uint32_t)(glb_K_value << 16) | (uint32_t)(glb_N_value);

	/* UART communication start. */
	TSB_UART0->TRANS = 0x00000002;


	Sendbuf = 'x';
	SendData(&Sendbuf , 1);		/* Return 'x' to HOST. */

	return;

}


/*
 *********************************************************************
 * NAME	: SendData
 *--------------------------------------------------------------------
 * DESCRIPTION	: Data transmission processing.
 *				  The number of bytes specified by 'nLen' is transmitted as data stored in 'pData'.
  * PARAMETER	: pData Transmit data storage pointer.
 *				  nLen  Number of transmission data bytes.
 *
 * RETURN VALUE	: None.
 *
 *********************************************************************
 */
void SendData(uint8_t *pData, uint32_t nLen)
{
	uint32_t Count;						/* Transmit data counter. */

	for (Count = 0; Count < nLen; Count++) {
		TSB_UART0->DR = pData[Count];			/* Store in transmission buffer. */

		for(;;) {
			if (NVIC_GetPendingIRQ(INTUART0TX_IRQn)) {
				break;							/* Transmit latch ON detection. */
			} else {
				/* Do Nothing */
			}
		}
		NVIC_ClearPendingIRQ(INTUART0TX_IRQn);	/* Transmit latch clear. */

	}

	return;
}


/*
 *********************************************************************
 * NAME	: RecvData
 *--------------------------------------------------------------------
 * DESCRIPTION	: Data reception processing.
 *				  Receives as many bytes as specified by 'nLen' and stores it in 'pData'.
 * PARAMETER	: pData Received data storage pointer.
 *				  nLen  Number of received data bytes.
 *
 * RETURN VALUE	: Number of received data bytes (0-bytes at reception error)
 *
 *********************************************************************
 */
uint32_t RecvData(uint8_t *pData, uint32_t nLen)
{
	uint32_t RecvNum = 0;					/* Number of bytes received. */
	uint32_t Count;
	uint32_t work;

	for(Count = 0; Count < nLen; Count++) {
		work = TSB_UART0->TRANS | 0x00000001;
		TSB_UART0->TRANS = work;				/* Receive permission. */

		for(;;) {								/* Waiting for reception. */
			if(NVIC_GetPendingIRQ(INTUART0RX_IRQn)) {
				break;							/* Receive latch ON detection. */
			}
			else{
				/* Do Nothing */
			}
		}
		NVIC_ClearPendingIRQ(INTUART0RX_IRQn);	/* Receive latch clear. */

		*pData = (uint8_t)(TSB_UART0->DR & 0x000000FF);		/* Received data storage. */
		pData++;
		RecvNum++;

		/* 'EOT': End of application transfer.(Because it is only 1-byte of data, stop receiving here.) */
		if (Recvbuf[0] == EOT) {
				break;
		} else {
			/* Do Nothing */
		}

		/* Error checking. */
		if((TSB_UART0->ERR & 0x1F) != COM_NO_ERR) {
			return RECV_ERR;
		} else {
			/* Do Nothing */
		}
	}

	return RecvNum;
}


/*
 *********************************************************************
 * NAME	: TimerStart
 *--------------------------------------------------------------------
 * DESCRIPTION	: T32A Timer-C operation started.
 * PARAMETER	: None.
 *
 * RETURN VALUE	: None.
 *
 *********************************************************************
 */
void TimerStart(void)
{

	/* Check Timer-C operation flag. */
	if ((TSB_T32A0->RUNC & T32A_RUNFLG_RUN) == T32A_RUNFLG_RUN) {
		do {
			TimerStop();
		} while ((TSB_T32A0->RUNC & T32A_RUNFLG_RUN) != T32A_RUNFLG_STOP);
	} else {
		/* Do Nothing */
	}

	/* Timer-C permission and count start. */
	TSB_T32A0->RUNC |= (T32A_COUNT_START | T32A_RUN_ENABLE);

	return;
}

/*
 *********************************************************************
 * NAME	: TimerStop
 *--------------------------------------------------------------------
 * DESCRIPTION	: T32A Timer-C operation stopped.
 * PARAMETER	: Ȃ
 *
 * RETURN VALUE	: Ȃ
 *
 *********************************************************************
 */
void TimerStop(void)
{

	/* Timer-C prohibition and count stop. */
	TSB_T32A0->RUNC = T32A_COUNT_STOP;

	return;
}


/**
 *  @} 
 */

#ifdef __cplusplus
}
#endif /* __cplusplus */

/* The End of com_control.c file */
