/** 
 *******************************************************************************
 * @file    main.c
 * @brief   ISP_BOOT Sample Application.
 * @version V1.1.0.0
 * $Date:: 2018-11-29
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LISENCE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2018 All rights reserved
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/**
 *  @addtogroup Sample_Appli
 *  @{
 */

/** 
 *  @addtogroup ISP_BOOT
 *  @{
 */

/** 
 *  @defgroup MAIN_PRIVATE  MAIN Private
 *  @brief    MAIN Private Group.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "stdio.h"

#include "system_TMPM3Hy.h"
#include "TMPM3HQ.h"

#include "txz_uart.h"
#include "txz_t32a.h"

#include "main.h"
#include "com_control.h"
#include "Flash_Erase.h"
#include "Flash_Write.h"


/*------------------------------------------------------------------------------*/
/*  Function Definitions                                                        */
/*------------------------------------------------------------------------------*/
void BootStop(void);
static void internal_hosc_enable(void);


/*------------------------------------------------------------------------------*/
/*  Variable Definitions                                                        */
/*------------------------------------------------------------------------------*/
uint8_t glb_CurCmd;		/* Command storage. */


/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{

	uint8_t		RcvData;		/* Received data storage. */
	uint32_t	RecvLen;		/* Number of bytes received. */

	uint8_t		erase_flag = 0;
	uint8_t		write_flag = 0;

	uint32_t	i;

    /*----------------------*/
    /* Initialize           */
    /*----------------------*/

	/* Internal high speed oscillation enabled */
	internal_hosc_enable();

	/* Peripheral circuit clock supply. */
	do {
		TSB_CG->FSYSENA = (0xFF03FFFFUL);				/* TSB_CG->FSYSENA : Supplied to T32A and Port. */
	} while (TSB_CG->FSYSENA != (0xFF03FFFFUL));		/* Read confirmation after writing. */

	do {
		TSB_CG->FSYSENB = (0xF00007E0UL);				/* TSB_CG->FSYSENB : Supplied to SIWDT and UART. */
	} while (TSB_CG->FSYSENB != (0xF00007E0UL));		/* Read confirmation after writing. */


	/* Port_PSW(SW3 : PV3) initialize */
	TSB_PV->IE = (0x00000008UL);

	/* Port_LED(LED0,1,3 : PK0-3) initialize */
	TSB_PK->CR = (0x000000B0UL);
	TSB_PK->DATA = (0x00000000UL);

	for (i = 0; i < 1000; i++){;}	/* Wait until the setting is reflected. */

	/* Application launched with SW3_OFF. */
	if ((TSB_PV->DATA & 0x00000008) == 0x000000008) {
		/* Return the port setting. */
		TSB_PV->IE = (0x00000000UL);
		TSB_PK->CR = (0x00000000UL);

		/* ROM jump */
		BOOT_boot();

	/* ISP_BOOT started SW3_ON for 2 seconds. */
	} else {
		for (i = 0; i < 0x00200000; i++) {	/* About 2 seconds confirmation. */
			if ((TSB_PV->DATA & 0x00000008) == 0x000000000) {
				/* Do Nothing */
			} else {
				/* Return the port setting. */
				TSB_PV->IE = (0x00000000UL);
				TSB_PK->CR = (0x00000000UL);

				/* ROM jump */
				BOOT_boot();
			}
		}

	}

	/* UART communication establishment */
	StartCom();

	/* LED0_ON */
	TSB_PK->DATA = (0x00000010UL);

    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/

	for (;;) {

		/* Wait for command reception. */
		RecvLen = RecvData(&RcvData , 1);

		if (RecvLen == RECV_ERR) {
			/* UART ERROR Flag detection. */
			for(;;) {;}		/* infinite loop */
		} else {
			/* Return the received command to the HOSt as it is. */
			glb_CurCmd = RcvData;
			SendData(&glb_CurCmd , 1);

			/* Command processing. */
			switch (glb_CurCmd) {

			case 't':	/* Command 't' : Application transfer and FLASH write. */
				/* FLASH Erase : PG2-PG7ABlock1-Block15 */
				erase_flag = Flash_Erase();
				if (erase_flag == 0) {
					/* FLASH Erase successful */
					SendACK();

					/* Write application to FLASH. */
					write_flag = Flash_Write();
					if (write_flag == 0) {
						/* Successfully written to FLASH. */
						SendACK();

						/* LED3_ON */
						TSB_PK->DATA = (0x00000080UL);

					} else {
						/* Failed to write to FLASH. */
						SendNAK();
					}

				} else {
					/* Failed to write to FLASH. */
					SendNAK();
				}

				break;

			case 's':	/* Command 's' : Application activation. */

				/* All LEDs are off. */
				TSB_PK->DATA = (0x00000000UL);

				/* Return the port setting. */
				TSB_PV->IE = (0x00000000UL);
				TSB_PK->CR = (0x00000000UL);

				/* ROM jump */
				BOOT_boot();

				break;

			case 'r':	/* Command 'r' : Reset System */

				NVIC_SystemReset();

				break;

			default:
				/* Other than command 't''s''r'. */
				/* Do Nothing */
				break;
			}
		}
	}


    /* dummy loop */
    while (1){;}
}


/*
 *********************************************************************
 * NAME	: BootStop
 *--------------------------------------------------------------------
 * DESCRIPTION	: Operation stop processing.
 * PARAMETER	: Ȃ
 *
 * RETURN VALUE	: Ȃ
 *
 *********************************************************************
 */
void BootStop(void)
{
	for(;;) {;}
}


/*--------------------------------------------------*/
/** 
  * @brief  Internal HOSC Enable
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void internal_hosc_enable(void)
{
	uint32_t work;

	/* [CGOSCCR]<IHOSC1EN> : HOSC Enable    */
	work = (uint32_t)(TSB_CG->OSCCR & ~CGOSCCR_IHOSC1EN_MASK);
	TSB_CG->OSCCR = (uint32_t)(work | CGOSCCR_IHOSC1EN_RW_ENABLE);

	return;
}


#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
/*        __nop();*/
		;
    }
}
#endif
/**
 *  @} 
 */ /* End of group GPIO_LED */

/**
 *  @} 
 */ /* End of group Sample_Appli */

#ifdef __cplusplus
}
#endif /* __cplusplus */
